function initHistoryChat()
	for i = 1, NUM_CHAT_WINDOWS, 1 do
		local chatFrame = getglobal("ChatFrame"..i);
		local chatFrameTab = getglobal("ChatFrame"..i.."Tab");
		local chatFrameName = chatFrame["name"];
		
		if (chatFrameTab:IsShown() or chatFrameTab:IsVisible()) then
			if (chatFrameName == "RageComic") then
				ChatFrame_RemoveAllMessageGroups(chatFrame);
				ChatFrame_RemoveAllChannels(chatFrame);
				ChatFrame_ReceiveAllPrivateMessages(chatFrame);
				--ChatFrame_ReceiveAllBNConversations(chatFrame);
				break;
			end
		end
	end
end

local function getRageComicHistoryChat()
	if (RageComicOptions["history"]["enabled"] == false) then
		return nil;
	end
	
	for i = 1, NUM_CHAT_WINDOWS, 1 do
		local chatFrame = getglobal("ChatFrame"..i);
		local chatFrameTab = getglobal("ChatFrame"..i.."Tab");
		local chatFrameName = chatFrame["name"];
		
		if (chatFrameTab:IsShown() or chatFrameTab:IsVisible()) then
			if (chatFrameName == "RageComic") then
				return chatFrame;
			end
		end
	end
	
	local rageChat = FCF_OpenNewWindow("RageComic");
	ChatFrame_RemoveAllMessageGroups(rageChat);
	ChatFrame_RemoveAllChannels(rageChat);
	ChatFrame_ReceiveAllPrivateMessages(rageChat);
	--ChatFrame_ReceiveAllBNConversations(rageChat);
	
	return rageChat;
end

function printMessage(msg)
	print("|cffc0c0c0RageComic|r: "..msg);
end
function printHistory(player, image, fromType)
	--printHistory(target, image, nil);
	--printHistory(sender, arg1, msgtype);
	
	local rageHistory = getRageComicHistoryChat();
	local msg;
	local size = 64;
	
	if (rageHistory ~= nil) then
		size = rageHistory:GetHeight() - 10;
	
		if (size > 100) then
			size = size / 2;
		end
	
		if (fromType == nil) then
			msg = "[To "..player.."] |T"..locationImages.."rage_"..image..".tga:"..size.."|t"
		else
			msg = "[From "..player.."] |T"..locationImages.."rage_"..image..".tga:"..size.."|t [over "..fromType.."]"
		end
	
		rageHistory:AddMessage(msg);

		if (fromType ~= nil and (not rageHistory:IsShown()) and (not rageHistory:IsVisible())) then
			FCF_StartAlertFlash(rageHistory);
		end
	end		
end

-- Prints the usage/help
function printUsage()
	printMessage("RageComic Quickhelp");
	printMessage("|cffc0c0c0===================|r");
	
	printMessage("/rc |cffc0c0c0Shows the UI|r");
	
	printMessage("/rc show <#|?> |cffc0c0c0Shows the comic number <#>, or random ?|r");
	printMessage("/rc move <start|stop> |cffc0c0c0Dis/Enables the repositioning|r");
	printMessage("/rc hide |cffc0c0c0Hides the comic immediately|r");
	
	printMessage("/rc set <x|y|size> <#> |cffc0c0c0Sets position and size|r");
	printMessage("/rc set amount <#> |cffc0c0c0Sets the amount of current rage comics|r");
	printMessage("/rc set timer <#> |cffc0c0c0Sets the timer for the comic|r");
	printMessage("/rc set location <map|chat> |cffc0c0c0Positions the comic over the minimap|r");
	
	printMessage("/rc send <guild||raid|party> <#|?> |cffc0c0c0Sends the comic number <#> or random ? to the channel|r");
	printMessage("/rc send <playername> <#|?> |cffc0c0c0Sends the comic number <#> or random ? to the player|r");
	
	printMessage("/rc mute <playername> |cffc0c0c0Comics sent by this play will be ignored|r");
	printMessage("/rc unmute <playername> |cffc0c0c0Comics sent by this player will be shown again|r");
	printMessage("/rc list |cffc0c0c0Lists all muted players|r");	
	
	printMessage("/rc <feedback|ping> |cffc0c0c0Checks all player in guild and raid for feedback|r");
end

local function splitVersionInfo(v)
	-- major and minor version number
	local major, minor = strsplit(".", v, 2);	
	-- release type "a" (alpha), "b" (beta), "" (release)
	local release = string.sub(v, string.len(v), string.len(v));
	
	-- if release
	if (tonumber(release) ~= nil) then
		release = "";
	end
	
	-- get number from minor part (minor + release)
	minor = string.sub(minor, 1, (string.len(minor) - string.len(release)));
	minor = tonumber(minor);
	
	major = tonumber(major);
	
	local returnValue = { major, minor, release };
	return returnValue;
end
function isNewerVersion(v)
	-- Version-Style: 0.1a
	local current = splitVersionInfo(version);
	local compare = splitVersionInfo(v);
	
	if (compare[1] > current[1]) then return true; end
	if (compare[1] == current[1] and compare[2] > current[2]) then return true; end
	if (compare[1] == current[1] and compare[2] == current[2]) then
		if (compare[3] == "" and current[3] ~= "") then
			return true;
		end
		if (compare[3]:byte() > current[3]:byte()) then
			return true;
		end
	end
	
	return false;
end