--------------------------------------------------------------------------------
-- Local functions
--------------------------------------------------------------------------------

local function initAddon()
	if (RageComicOptions == nil) then
		RageComicOptions = {};
	end

	if (RageComicOptions["blacklist"] == nil) then
		RageComicOptions["blacklist"] = {};
	end
	
	-- Init history
	initHistoryChat();
	
	-- Load display frame
	if (RageComicFrameDisplay["isInit"] == false) then
		RageComicFrameDisplay.doInit();
	end
	-- Load UI frame
	if (RageComicFrameUI["isInit"] == false) then
		RageComicFrameUI.doInit();
	end
end
local function storeOptions()
	-- Store display options
	RageComicFrameDisplay.storeOptions();
	-- Store UI options
	RageComicFrameUI.storeOptions();
end

--------------------------------------------------------------------------------
-- Event processing
--------------------------------------------------------------------------------

local function processVersionRequest(cmd, v, sender)
	if (isNewerVersion(v)) then
		printMessage("Newer version available ("..v..")!");
	end
	
	-- if version request: send result
	if (cmd == "version") then
		SendAddonMessage(prefixAddon, "version_result "..version, "WHISPER", sender);
	end
end
local function processVersionCheckup()
	SendAddonMessage(prefixAddon, "version "..version, "GUILD", nil);
	SendAddonMessage(prefixAddon, "version "..version, "RAID", nil);
end

local function processFeedbackRequest(cmd, arg1, arg2, sender)
	if (cmd == "feedback") then
		local blacklisted = "";
		
		if (RageComicOptions["blacklist"][sender] ~= nil and RageComicOptions["blacklist"][sender] == true) then
			blacklisted = " blacklisted";
		end
		
		SendAddonMessage(prefixAddon, "feedback_result "..version.." "..blacklisted, "WHISPER", sender);
	end
	
	if (cmd == "feedback_result") then
		local v = "";
		local b = "";

		if (arg1 ~= nil and arg1 ~= "") then v = " with version "..arg1; end
		if (arg2 ~= nil and arg2 ~= "") then b = " (you are blacklisted)"; end
		
		printMessage("Feedback from "..sender..v..b);
	end
end
local function processCommandFeedback()
	SendAddonMessage(prefixAddon, "feedback", "GUILD", nil);
	SendAddonMessage(prefixAddon, "feedback", "RAID", nil);
end

local function processMute(cmd, player)
	if (cmd == "mute") then
		if (player ~= nil) then			
			if (RageComicOptions["blacklist"][player] ~= nil and RageComicOptions["blacklist"][player] == true) then
				printMessage("The player "..player.." is already muted");
			else
				RageComicOptions["blacklist"][player] = true;
				printMessage("Muted "..player.." - you will not get any rage comics from this player");
			end
			
			return;
		end
	elseif (cmd == "unmute") then
		if (player ~= nil) then
			if (RageComicOptions["blacklist"][player] == nil or RageComicOptions["blacklist"][player] == false) then
				printMessage("The player "..player.." is not muted.");
			else
				RageComicOptions["blacklist"][player] = false;
				printMessage("Unmuted "..player.." - rage comics for everyone!");
			end
			
			return;
		end
	elseif (cmd == "list") then		
		local playersMuted = false;
		printMessage("Muted players:");
		
		for a,b in pairs(RageComicOptions["blacklist"]) do 
			if (b == true) then
				printMessage("- "..a);
				playersMuted = true;
			end
		end
		
		if (playersMuted == false) then
			printMessage("<none>");
		end
		
		return;
	end
	
	printUsage();
end

--------------------------------------------------------------------------------
-- Events / Communication
--------------------------------------------------------------------------------

local RageComicEventFrame 			= CreateFrame("Frame");
local RageComicCommunicationFrame 	= CreateFrame("Frame");
RageComicCommunicationFrame:RegisterEvent("CHAT_MSG_ADDON");

function RageComicEventFrame:OnEvent(event, addon)
	if (event == "ADDON_LOADED" and addon == "RageComic") then
		initAddon();
		processVersionCheckup();
	elseif event == "PLAYER_LOGOUT" then
		storeOptions();
	end
end
local function handleCommunication(self, event, prefix, message, msgtype, sender)
	if (event == "CHAT_MSG_ADDON") then
		if (prefix == prefixAddon) then
			local cmd, arg1, arg2, arg3, arg4 = strsplit(" ", message);
									
			if (cmd == "show") then
				if (RageComicOptions["blacklist"][sender] == nil or RageComicOptions["blacklist"][sender] == false) then		
					if (sender ~= UnitName("player")) then
						--printMessage("[From "..sender.."] |T"..locationImages.."rage_"..arg1..".tga:60|t [over "..msgtype.."]");
						printHistory(sender, arg1, msgtype);
					end
					processCommandShow(arg1, sender);
				end
			elseif (cmd == "version" or cmd == "version_result") then
				processVersionRequest(cmd, arg1, sender);
			elseif (cmd == "feedback" or cmd == "feedback_result") then
				processFeedbackRequest(cmd, arg1, arg2, sender)
			end
		end
	end
end

--------------------------------------------------------------------------------
-- Events
--------------------------------------------------------------------------------

RageComicCommunicationFrame:SetScript("OnEvent", handleCommunication);

RageComicEventFrame:SetScript("OnEvent", RageComicEventFrame.OnEvent);
RageComicEventFrame:RegisterEvent("ADDON_LOADED");
RageComicEventFrame:RegisterEvent("PLAYER_LOGOUT");

--------------------------------------------------------------------------------
-- Slash commands
--------------------------------------------------------------------------------
local function handlerSlashCommands(msg, editbox)
	local cmd, arg1, arg2, arg3, arg4 = strsplit(" ", msg);

	if (cmd == "") then	
		RageComicFrameUI.showFrame();
		return;
	elseif (cmd == "show") then
		processCommandShow(arg1, nil);
		return;
	elseif (cmd == "move") then
		processCommandMove(arg1);
		return;
	elseif (cmd == "set") then
		processCommandSet(arg1, arg2);
		return;
	elseif (cmd == "mute" or cmd == "unmute" or cmd == "list") then
		processMute(cmd, arg1);
		return;
	elseif (cmd == "send") then
		processCommandSend(arg1, arg2);
		return;
	elseif (cmd == "feedback" or cmd == "ping") then
		processCommandFeedback();
		return;
	elseif (cmd == "debug") then
		
		print(ChatFrame1:GetHeight());
		
		return;
	end

	printUsage();
end

SLASH_RAGECOMIC1, SLASH_RAGECOMIC2 = '/ragecomic', '/rc';
SlashCmdList["RAGECOMIC"] = handlerSlashCommands;