--------------------------------------------------------------------------------
-- RageComicFrame Display
--------------------------------------------------------------------------------

RageComicFrameDisplay = CreateFrame("Frame");

local timer =
{
	["enabled"] = false,
    ["current"] = 0
}

--------------------------------------------------------------------------------
-- Local functions
--------------------------------------------------------------------------------

local function restartTimer()
	timer["enabled"] = true;
    timer["current"] = 0;
end
local function stopTimer()
	timer["enabled"] = false;
    timer["current"] = 0;
end

local function initOptions()
	if (RageComicOptions == nil) then
		RageComicOptions = {};
	end
	if (RageComicOptions["display"] == nil) then
		RageComicOptions["display"] = {};
	end
	
	if (RageComicOptions["display"]["x"] == nil) then
		local left = (UIParent:GetWidth() / 2) - 100;
		if (left < 0) then left = 0; end;
	
		RageComicOptions["display"]["x"] = left;
	end
	if (RageComicOptions["display"]["y"] == nil) then
		local bottom = (UIParent:GetHeight() / 2) - 100;
		if (bottom < 0) then bottom = 0; end;
	
		RageComicOptions["display"]["y"] = bottom;
	end
	if (RageComicOptions["display"]["size"] == nil) then
		RageComicOptions["display"]["size"] = 200;
	end
	if (RageComicOptions["display"]["timer"] == nil) then
		RageComicOptions["display"]["timer"] = 3;
	end
	
	-- Update the amount of comics
	RageComicOptions["display"]["amount"] = 80;
end

local function updateFramePosition()
	RageComicFrameDisplay:SetSize(RageComicOptions["display"]["size"], RageComicOptions["display"]["size"]);
	RageComicFrameDisplay:SetPoint("BOTTOMLEFT", RageComicOptions["display"]["x"], RageComicOptions["display"]["y"]);
end

local function setImage(image)

	local imgLocation = locationImages.."rage_"..image..".tga";

	local backdrop = {
		-- path to the background texture
		bgFile = imgLocation,  
		-- path to the border texture
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		-- true to repeat the background texture to fill the frame, false to scale it
		tile = false,
		-- size (width or height) of the square repeating background tiles (in pixels)
		tileSize = 32,
		-- thickness of edge segments and square size of edge corners (in pixels)
		edgeSize = 2,
		-- distance from the edges of the frame to those of the background texture (in pixels)
		insets = {
			left = 2,
			right = 2,
			top = 2,
			bottom = 2
		}
	}
	RageComicFrameDisplay:SetBackdrop(backdrop);
end
local function setSender(sender)
	if (sender ~= nil) then
		RageComicFrameDisplay["text"]:SetText("by "..sender);
	else
		RageComicFrameDisplay["text"]:SetText(nil);
	end
end
local function showFrame(image, sender)
	restartTimer();
	
	setSender(sender);
	setImage(image);
	RageComicFrameDisplay:Show();
end
local function hideFrame()
	stopTimer();
	RageComicFrameDisplay:Hide();
end

local function startMovingFrame()
	setSender(nil);
	setImage("default");
	RageComicFrameDisplay:Show();
	
	RageComicFrameDisplay:SetMovable(true);
	RageComicFrameDisplay:EnableMouse(true);
	
	printMessage("Drag the frame to wherever you like and exit moving status afterwards (/rc move stop)");
end
local function stopMovingFrame()
	RageComicFrameDisplay:SetMovable(false);
	RageComicFrameDisplay:EnableMouse(false);
	
	hideFrame();
end

--------------------------------------------------------------------------------
-- Frame layout
--------------------------------------------------------------------------------

local function initTextArea()
	RageComicFrameDisplay["text"] = RageComicFrameDisplay:CreateFontString();
	local RageComicText = RageComicFrameDisplay["text"];
	RageComicText:SetPoint("BOTTOMLEFT", 5, 5);
	RageComicText:SetFont("Fonts\\FRIZQT__.TTF", 10, nil);
	RageComicText:SetTextColor(0, 0, 0);
end

--------------------------------------------------------------------------------
-- Global functions
--------------------------------------------------------------------------------

function processCommandShow(image, sender)
	if (image == "?") then
		image = math.random(0, tonumber(RageComicOptions["display"]["amount"]));
	end
	
	if (tonumber(image) ~= nil and tonumber(image) >= 0 and tonumber(image) <= tonumber(RageComicOptions["display"]["amount"])) then
		showFrame(image, sender);
		return;
	else
		printMessage("Can not display rage comic "..image);
	end
	
	printUsage();
end

function processCommandSet(key, value)
	if (key == "timer") then
		if (tonumber(value) ~= nil) then
			RageComicOptions["display"]["timer"] = tonumber(value);
			printMessage("Set timer to "..value);
			return;
		end
	elseif (key == "x") then
		if (tonumber(value) ~= nil) then
			RageComicOptions["display"]["x"] = tonumber(value);
			updateFramePosition();
			printMessage("Set position x to "..value);
			return;
		end
	elseif (key == "y") then
		if (tonumber(value) ~= nil) then
			RageComicOptions["display"]["y"] = tonumber(value);
			updateFramePosition();
			printMessage("Set position y to "..value);
			return;
		end
	elseif (key == "size") then
		if (tonumber(value) ~= nil) then
			RageComicOptions["display"]["size"] = tonumber(value);
			updateFramePosition();
			printMessage("Set width and height to "..value);
			return;
		end
	elseif (key == "amount") then
		if (tonumber(value) ~= nil) then
			RageComicOptions["display"]["amount"] = tonumber(value);
			printMessage("Set amount of comics to "..value);
			return;
		end
	elseif (key == "location") then
		if (value == "map") then
			local tempFrame = CreateFrame("Frame");
			tempFrame:SetAllPoints("Minimap");
			
			local height = tempFrame:GetHeight();
			local width = tempFrame:GetWidth();
			local x = tempFrame:GetLeft();
			local y = tempFrame:GetBottom();
		
			RageComicFrameDisplay:SetSize(height, height);
			RageComicFrameDisplay:SetPoint("BOTTOMLEFT", x, y);
		
			printMessage("Set position and size to minimap");
			return;
		elseif (value == "chat") then
			local tempFrame = CreateFrame("Frame");
			tempFrame:SetAllPoints(ChatFrame1);
						
			local height = tempFrame:GetHeight();
			local width = tempFrame:GetWidth();
			local x = tempFrame:GetLeft() + (width - height);
			local y = tempFrame:GetBottom();
			
			RageComicFrameDisplay:SetSize(height, height);
			RageComicFrameDisplay:SetPoint("BOTTOMLEFT", x, y);
			
			printMessage("Set position and size to chat");
			return;
		end
		
		-- FIXME: Needed?
		RageComicFrameDisplay.storeOptions();
	end
	
	printUsage();
end

function processCommandMove(movetype)
	if (movetype == "start") then
		startMovingFrame();
		return;
	elseif (movetype == "stop") then
		stopMovingFrame();
		return;
	end
	
	printUsage();
end

--------------------------------------------------------------------------------
-- Events
--------------------------------------------------------------------------------

local function handlerOnUpdate(self, elapsed)
	if (timer["enabled"]) then
		timer["current"] = timer["current"] + elapsed;
		
		if (timer["current"] >= RageComicOptions["display"]["timer"]) then
			hideFrame();
		end
	end
end

RageComicFrameDisplay:RegisterForDrag("LeftButton");
RageComicFrameDisplay:SetScript("OnDragStart", RageComicFrameDisplay.StartMoving);			-- unused?
RageComicFrameDisplay:SetScript("OnDragStop", RageComicFrameDisplay.StopMovingOrSizing);		-- unused?

RageComicFrameDisplay:SetScript("OnUpdate", handlerOnUpdate); -- Timer

--------------------------------------------------------------------------------
-- Initialization
--------------------------------------------------------------------------------

RageComicFrameDisplay["isInit"] = false;
RageComicFrameDisplay["doInit"] = function()
	RageComicFrameDisplay["isInit"] = true;

	initOptions();
		
	updateFramePosition();
	
	RageComicFrameDisplay:SetFrameStrata("Tooltip");
	RageComicFrameDisplay:Raise();

	initTextArea();
	
	RageComicFrameDisplay:Hide();
end;
RageComicFrameDisplay["storeOptions"] = function()
	if (RageComicOptions["display"] == nil) then
		RageComicOptions["display"] = {};
	end
	
	RageComicOptions["display"]["x"] 	= RageComicFrameDisplay:GetLeft();
	RageComicOptions["display"]["y"] 	= RageComicFrameDisplay:GetBottom();
	RageComicOptions["display"]["size"] = RageComicFrameDisplay:GetWidth();
	
	printMessage("DEBUG - Storing FrameDisplay "..RageComicFrameDisplay:GetLeft().." "..RageComicFrameDisplay:GetBottom());
end;