--------------------------------------------------------------------------------
-- RageComicFrame UI
--------------------------------------------------------------------------------

RageComicFrameUI = CreateFrame("Frame");

local width 		= 800;
local height 		= 202;

local comic_size	= 100;
local border_width	= 8;
local title_height 	= 20;

local content_width = width - border_width - border_width;

local itemsTarget = {
	["Guild"] 	= "guild",
	["Raid"] 	= "raid",
	["Party"] 	= "party",
	["Player"] 	= "whisper";
}

--------------------------------------------------------------------------------
-- Local functions
--------------------------------------------------------------------------------

local function getTargetType()
	return UIDropDownMenu_GetSelectedValue(RageComicFrameUI["target_type"]);
end
local function getTargetName()
	return RageComicFrameUI["target_name"]:GetText();
end
local function processShow(image)
	local target_type = getTargetType();
	local target_name = getTargetName();
	
	if (target_type == "whisper" or target_type == "player") then
		target_type = target_name;
	end
	
	if (target_type == nil or target_type == "") then
		printMessage("You need to specify a player name");
	else
		processCommandSend(target_type, image);
	end	
end

local function initOptions()
	if (RageComicOptions == nil) then
		RageComicOptions = {};
	end
	if (RageComicOptions["ui"] == nil) then
		RageComicOptions["ui"] = {};
	end
	
	if (RageComicOptions["ui"]["x"] == nil) then
		RageComicOptions["ui"]["x"] = 0;
	end
	if (RageComicOptions["ui"]["y"] == nil) then
		RageComicOptions["ui"]["y"] = 0;
	end
	if (RageComicOptions["ui"]["type"] == nil) then
		RageComicOptions["ui"]["type"] = itemsTarget["Guild"];
	end
	if (RageComicOptions["ui"]["player"] == nil) then
		RageComicOptions["ui"]["player"] = "";
	end
	if (RageComicOptions["ui"]["scrollbar"] == nil) then
		RageComicOptions["ui"]["scrollbar"] = 0;
	end
	
	if (RageComicOptions["history"] == nil) then
		RageComicOptions["history"] = {};
	end
	if (RageComicOptions["history"]["enabled"] == nil) then
		RageComicOptions["history"]["enabled"] = true;
	end
end

--------------------------------------------------------------------------------
-- Frame Layout
--------------------------------------------------------------------------------

local function fillScrollArea()
	local ScrollContent = RageComicFrameUI["area_content_comics"];
	
	local space = 5;
	local scroll_width = 0;
	local comics = {};
	
	-- Create comics
	for i = 0, tonumber(RageComicOptions["display"]["amount"]), 1 do
		comics[i] = CreateFrame("Button", "RageComic"..i, ScrollContent);
		comics[i]:SetSize(comic_size, comic_size);
		comics[i]:RegisterForClicks("LeftButtonUp");
		comics[i]:SetScript("OnDoubleClick", function(self, arg1) processShow(i); end);
		
		-- image
		local texture = comics[i]:CreateTexture();
		texture:SetTexture(locationImages.."rage_"..i..".tga");
		texture:SetAllPoints(comics[i]);
		
		-- label
		local id = comics[i]:CreateFontString();
		id:SetFont("Fonts\\FRIZQT__.TTF", 10, nil);
		id:SetPoint("BOTTOMRIGHT", 0, 0);
		id:SetTextColor(0, 0, 0);
		id:SetText(""..i);
		
		-- position
		if (i == 0) then
			comics[i]:SetPoint("TOPLEFT", 0, 0);
			scroll_width = scroll_width + comic_size;
		else
			comics[i]:SetPoint("TOPLEFT", comics[i - 1], "TOPRIGHT", space, 0);
			scroll_width = scroll_width + comic_size + space;
		end
	end

	return scroll_width;
end
local function initScrollArea()
	local ContentArea = RageComicFrameUI["area_content"];
	
	local ScrollFrame = CreateFrame("ScrollFrame", "ScrollFrame", ContentArea);
	ScrollFrame:SetSize(content_width, comic_size);
	ScrollFrame:SetPoint("TOP", ContentArea, "TOP", 0, -50);
	ScrollFrame:EnableMouseWheel(true);
	
	RageComicFrameUI["area_content_comics"] = CreateFrame("Frame", "ScrollContent", ScrollFrame);
	local ScrollContent = RageComicFrameUI["area_content_comics"];
		
	local ScrollContentBackground = ScrollContent:CreateTexture(nil, "BACKGROUND");
	ScrollContentBackground:SetAllPoints(true);
	ScrollContentBackground:SetColorTexture(1.0, 1.0, 1.0, 0.5);
	
	ScrollFrame:SetScrollChild(ScrollContent);
	 	
	local comic_width = fillScrollArea();
	local scrollMax = comic_width - content_width;
	
	-- Set the size of the scroll child 
	ScrollContent:SetSize(comic_width, comic_size);

	RageComicFrameUI["area_content_scrollbar"] = CreateFrame("Slider", "Scrollbar", ScrollFrame);
	local Scrollbar = RageComicFrameUI["area_content_scrollbar"];
	Scrollbar:SetOrientation("HORIZONTAL");
	Scrollbar:SetSize(ScrollFrame:GetWidth(), 16);
	Scrollbar:SetPoint("TOP", ScrollFrame, "BOTTOM", 0, 0);
	Scrollbar:SetMinMaxValues(0, scrollMax);
	Scrollbar:SetScript("OnValueChanged", function(self) ScrollFrame:SetHorizontalScroll(self:GetValue()); end);
	Scrollbar:SetValue(RageComicOptions["ui"]["scrollbar"]);
	
	local ScrollbarBackground = Scrollbar:CreateTexture(nil, "BACKGROUND");
	ScrollbarBackground:SetAllPoints(true);
	ScrollbarBackground:SetColorTexture(1.0, 1.0, 1.0, 0.5);
	
	local ScrollbarIcon = Scrollbar:CreateTexture(nil, "OVERLAY");
	ScrollbarIcon:SetTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
	ScrollbarIcon:SetSize(25, 25);
	
	Scrollbar:SetThumbTexture(ScrollbarIcon);
	
	-- Enable mousewheel scrolling
	ScrollFrame:SetScript("OnMouseWheel", function(self, delta)
		  local current = Scrollbar:GetValue();
		   
		  if IsShiftKeyDown() and (delta > 0) then
			 Scrollbar:SetValue(0);
		  elseif IsShiftKeyDown() and (delta < 0) then
			 Scrollbar:SetValue(scrollMax);
		  elseif (delta < 0) and (current < scrollMax) then
			 Scrollbar:SetValue(current + comic_size);
		  elseif (delta > 0) and (current > 1) then
			 Scrollbar:SetValue(current - comic_size);
		  end
	end);
end
local function initComboArea()
	local ContentArea = RageComicFrameUI["area_content"];
	
	local LabelTarget = ContentArea:CreateFontString();
	LabelTarget:SetFont("Fonts\\FRIZQT__.TTF", 10, nil);
	LabelTarget:SetPoint("TOPLEFT", ContentArea, "TOPLEFT", 10, -20);
	LabelTarget:SetTextColor(1, 1, 1);
	LabelTarget:SetText("Target: ");	
	
	RageComicFrameUI["target_type"] = CreateFrame("Button", "ComboTarget", ContentArea, "UIDropDownMenuTemplate");
	local ComboTarget = RageComicFrameUI["target_type"];
	ComboTarget:SetPoint("LEFT", LabelTarget, "RIGHT", 10, 0);	
	--ComboTarget:Show()
	 
	RageComicFrameUI["target_name"] = CreateFrame("EditBox", "InputTarget", ContentArea, "InputBoxTemplate");
	local InputTarget = RageComicFrameUI["target_name"];
    InputTarget:SetHeight(ComboTarget:GetHeight());
	InputTarget:SetWidth(200);
    InputTarget:SetPoint("LEFT", ComboTarget, "RIGHT", 10, 0);
	InputTarget:SetAutoFocus(false);
	InputTarget:SetScript("OnEnterPressed", function(self) self:ClearFocus(); end);
	InputTarget:SetText(RageComicOptions["ui"]["player"]);
	
	-- Initialize ComboBox
	UIDropDownMenu_Initialize(ComboTarget, function(self, level)
		--local info = UIDropDownMenu_CreateInfo();
		
		for k, v in pairs(itemsTarget) do
			local info = UIDropDownMenu_CreateInfo();
			info.text = k;
			info.value = v;
			info.func = function(self)
				UIDropDownMenu_SetSelectedID(ComboTarget, self:GetID());
				UIDropDownMenu_SetSelectedValue(ComboTarget, self.value);
			end
			
			UIDropDownMenu_AddButton(info, level);
		end
	end)

	UIDropDownMenu_SetWidth(ComboTarget, 100);
	UIDropDownMenu_SetButtonWidth(ComboTarget, 132);
	UIDropDownMenu_JustifyText(ComboTarget, "LEFT");
	UIDropDownMenu_SetSelectedValue(ComboTarget, RageComicOptions["ui"]["type"]);
end

local function initContentArea()
	RageComicFrameUI["area_content"] = CreateFrame("Frame", "ContentArea", RageComicFrameUI);
	local ContentArea = RageComicFrameUI["area_content"];
	ContentArea:SetSize(content_width, height - RageComicFrameUI["area_title"]:GetHeight() - border_width - border_width);
	ContentArea:SetPoint("TOP", RageComicFrameUI["area_title"], "BOTTOM");
	
	initComboArea();
	initScrollArea();
end
local function initTitleArea()
	RageComicFrameUI["area_title"] = CreateFrame("Frame", "TitleArea", RageComicFrameUI);
	local TitleArea = RageComicFrameUI["area_title"];
	TitleArea:SetSize(content_width, title_height);
	TitleArea:SetPoint("TOP", 0, - (border_width));

	local TitleText = TitleArea:CreateFontString();
	TitleText:SetFont("Fonts\\FRIZQT__.TTF", title_height * 0.75, nil);
	TitleText:SetPoint("CENTER", 0, 0);
	TitleText:SetTextColor(1, 1, 1);
	TitleText:SetText("RageComic User Interface");	
	
	local TitleSeparator = TitleArea:CreateTexture();
	TitleSeparator:SetSize(content_width, 2);
	TitleSeparator:SetPoint("BOTTOM", 0, 0);
	TitleSeparator:SetColorTexture(1, 1, 1, 0.5);

	-- DragArea
	RageComicFrameUI["area_title_drag"] = RageComicFrameUI:CreateTitleRegion();
	local DragRegion = RageComicFrameUI["area_title_drag"];
	DragRegion:SetAllPoints(TitleArea);
	
	-- Close Button
	local ButtonClose = CreateFrame("Button", "ButtonClose", RageComicFrameUI, "UIPanelCloseButton");
	ButtonClose:SetSize(title_height, title_height);
	ButtonClose:SetPoint("TOPRIGHT", - (border_width), - (border_width));
end

--------------------------------------------------------------------------------
-- Global functions
--------------------------------------------------------------------------------

RageComicFrameUI["showFrame"] = function()
	RageComicFrameUI:Show();
	RageComicFrameUI["area_title_drag"]:SetPoint("TOPLEFT");
end

function processCommandSend(target, image)
	if (image == "?") then
		image = math.random(0, tonumber(RageComicOptions["display"]["amount"]));
	end
	
	if (tonumber(image) ~= nil) then
		if (target == "guild") then
			--printMessage("[To GUILD] |T"..locationImages.."rage_"..image..".tga:60|t");
			printHistory(target, image, nil);
			SendAddonMessage(prefixAddon, "show "..image, "GUILD", nil);
			return;
		elseif (target == "party") then
			--printMessage("[To PARTY] |T"..locationImages.."rage_"..image..".tga:60|t");
			printHistory(target, image, nil);
			SendAddonMessage(prefixAddon, "show "..image, "PARTY", nil);
			return;
		elseif (target == "raid") then
			--printMessage("[To RAID] |T"..locationImages.."rage_"..image..".tga:60|t");
			printHistory(target, image, nil);
			SendAddonMessage(prefixAddon, "show "..image, "RAID", nil);
			return;
		else
			--printMessage("[To "..target.."] |T"..locationImages.."rage_"..image..".tga:60|t");
			printHistory(target, image, nil);
			processCommandShow(image, nil);
			SendAddonMessage(prefixAddon, "show "..image, "WHISPER", target);
			return;
		end
	end
	
	printUsage();
end

--------------------------------------------------------------------------------
-- Initialization
--------------------------------------------------------------------------------

RageComicFrameUI["isInit"] = false;
RageComicFrameUI["doInit"] = function()
	RageComicFrameUI["isInit"] = true;

	initOptions();
	
	local backdrop = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  	-- path to the background texture
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",		-- path to the border texture
		tile = false,													-- true to repeat the background texture to fill the frame, false to scale it
		tileSize = 32,													-- size (width or height) of the square repeating background tiles (in pixels)
		edgeSize = border_width * 2,									-- thickness of edge segments and square size of edge corners (in pixels)
		insets = { left = 2, right = 2, top = 2, bottom = 2 }			-- distance from the edges of the frame to those of the background texture (in pixels)
	}
	
	RageComicFrameUI:SetBackdrop(backdrop);
	
	RageComicFrameUI:SetSize(width, height);
	RageComicFrameUI:SetPoint("BOTTOMLEFT", RageComicOptions["ui"]["x"], RageComicOptions["ui"]["y"]);
	RageComicFrameUI:SetFrameStrata("HIGH");
	
	-- FIXME: Needed?	
	--RageComicFrameUI:SetMovable(true);
	RageComicFrameUI:EnableMouse(true);
	
	initTitleArea();
	initContentArea();
	
	RageComicFrameUI:Hide();
end;
RageComicFrameUI["storeOptions"] = function()
	if (RageComicOptions["ui"] == nil) then
		RageComicOptions["ui"] = {};
	end
	
	RageComicOptions["ui"]["x"] = RageComicFrameUI:GetLeft();
	RageComicOptions["ui"]["y"] = RageComicFrameUI:GetBottom();
	RageComicOptions["ui"]["type"] = getTargetType();
	RageComicOptions["ui"]["player"] = getTargetName();
	RageComicOptions["ui"]["scrollbar"] = RageComicFrameUI["area_content_scrollbar"]:GetValue();
	
	if (RageComicOptions["history"] == nil) then
		RageComicOptions["history"] = {};
	end
	-- TODO
	RageComicOptions["history"]["enabled"] = true;
end;